@@ -74,7 +74,7 @@ class Link(ABC):
 
         return None
 
-    def apply(self):
+    def apply(self, do_apply):
         # lookup for attributes requiring a interface index
         for attr in self.attr_idx:
             if attr in self.settings:
@@ -83,26 +83,46 @@ class Link(ABC):
         if self.idx is not None:
             self.iface = next(iter(ipr.get_links(self.idx)), None)
 
-            if self.cap_create and self.get_if_attr('kind') != self.settings['kind']:
-                self.recreate()
-            else:
-                self.update()
-        else:
-            self.create()
+            # check for ifname collisions
+            idx = next(iter(ipr.link_lookup(ifname=self.settings['ifname'])), None)
+            if idx is not None and idx != self.idx and do_apply:
+                try:
+                    ipr.link('set', index=idx, state='down')
+                    ipr.link('set', index=idx, ifname='{}!'.format(self.settings['ifname']))
+                except NetlinkError as err:
+                    logger.warning('renaming link {} failed: {}'.format(
+                        self.settings['ifname'], err.args[1]))
 
-    def create(self, oper="add"):
+            if self.cap_create and self.get_if_attr('kind') != self.settings['kind']:
+                self.recreate(do_apply)
+            else:
+                self.update(do_apply)
+        else:
+            self.create(do_apply)
+
+    def create(self, do_apply, oper="add"):
         logger.info(oper, extra={'iface': self.settings['ifname'], 'style': LogStyle.CHG})
 
         logger.debug("ip link add: {}".format( " ".join("{}={}".format(k, v) for k,v in self.settings.items()) ))
-        ipr.link('add', **(self.settings))
+        if do_apply:
+            try:
+                ipr.link('add', **(self.settings))
+            except NetlinkError as err:
+                logger.warning('adding link {} failed: {}'.format(
+                    self.settings['ifname'], err.args[1]))
 
-    def recreate(self):
+    def recreate(self, do_apply):
         logger.debug('has wrong link kind %s, removing', self.settings['kind'], extra={'iface': self.settings['ifname']})
-        ipr.link('del', index=self.idx)
+        if do_apply:
+            try:
+                ipr.link('del', index=self.idx)
+            except NetlinkError as err:
+                logger.warning('removing link {} failed: {}'.format(
+                    self.settings['ifname'], err.args[1]))
         self.idx = None
-        self.create("replace")
+        self.create(do_apply, "replace")
 
-    def update(self):
+    def update(self, do_apply):
         logger.debug('checking', extra={'iface': self.settings['ifname']})
 
         old_state = self.iface['state']
@@ -117,12 +137,25 @@ class Link(ABC):
             logger.debug('needs to be configured', extra={'iface': self.settings['ifname']})
             if old_state:
                 logger.debug('shutting down', extra={'iface': self.settings['ifname']})
-                ipr.link('set', index=self.idx, state='down')
+                if do_apply:
+                    try:
+                        ipr.link('set', index=self.idx, state='down')
+                    except NetlinkError as err:
+                        logger.warning('shutting down link {} failed: {}'.format(
+                            self.settings['ifname'], err.args[1]))
                 if not 'state' in self.settings:
                     self.settings['state'] = 'up'
 
-            logger.info('change', extra={'iface': self.settings['ifname'], 'style': LogStyle.CHG})
-            ipr.link('set', index=self.idx, **(self.settings))
+            if self.get_if_attr('ifname') == self.settings['ifname']:
+                logger.info('change', extra={'iface': self.settings['ifname'], 'style': LogStyle.CHG})
+            else:
+                logger.info('change (was {})'.format(self.get_if_attr('ifname')), extra={'iface': self.settings['ifname'], 'style': LogStyle.CHG})
+            if do_apply:
+                try:
+                    ipr.link('set', index=self.idx, **(self.settings))
+                except NetlinkError as err:
+                    logger.warning('updating link {} failed: {}'.format(
+                        self.settings['ifname'], err.args[1]))
         else:
             logger.info('ok', extra={'iface': self.settings['ifname'], 'style': LogStyle.OK})
 
