@@ -1,6 +1,7 @@
 from libifstate.util import logger
 from abc import ABC, abstractmethod
 
+
 class IfAttrList(list):
     def __init__(self, l):
         for i, v in enumerate(l):
@@ -10,6 +11,7 @@ class IfAttrList(list):
                 l[i] = IfAttrList(v)
         super().__init__(l)
 
+
 class IfAttrDict(dict):
     def __init__(self, d):
         for k, v in d.items():
@@ -24,11 +26,13 @@ class IfAttrDict(dict):
         #     print("booom")
         return super().__getitem__(key)
 
+
 class Parser(ABC):
     _default_ifstates = {
         'ignore': {
             'ipaddr': [
                 'fe80::/10',
+                'ff00::/8',
             ],
             'ifname': [
                 r'^docker\d+',
@@ -40,7 +44,10 @@ class Parser(ABC):
             ],
             'routes': {
                 'protos': [1, 2, 8, 9, 10, 11, 12, 13, 14, 15, 16, 42, 186, 187, 188, 189, 192],
-            }
+            },
+            'rules': {
+                'protos': [1, 2, 8, 9, 10, 11, 12, 13, 14, 15, 16, 42, 186, 187, 188, 189, 192],
+            },
         },
         'interfaces': {}
     }
@@ -50,5 +57,16 @@ class Parser(ABC):
         self.ifstate = {}
         pass
 
+    def merge(self, a, b):
+        for key in b:
+            if key in a:
+                if isinstance(a[key], dict) and isinstance(b[key], dict):
+                    self.merge(a[key], b[key])
+                else:
+                    a[key] = b[key]
+            else:
+                a[key] = b[key]
+        return a
+
     def config(self):
-        return IfAttrDict({**self._default_ifstates, **self.ifstates})
+        return IfAttrDict(self.merge(self._default_ifstates, self.ifstates))
