@@ -2,24 +2,36 @@ from libifstate.util import logger, ipr, LogStyle
 from libifstate.exception import RouteDupblicate
 from ipaddress import ip_address, ip_network
 from pyroute2.netlink.exceptions import NetlinkError
+from pyroute2.netlink.rtnl.fibmsg import FR_ACT_VALUES
 import collections.abc
 from glob import glob
 import os
 import re
 import sys
+import socket
 from socket import AF_INET, AF_INET6
 
-def route_matches(r1, r2, fields=('dst', 'metric', 'proto'), verbose=False):
+
+def route_matches(r1, r2, fields=('dst', 'metric', 'proto'), verbose=False, indent=""):
+    return _matches(r1, r2, fields, verbose, indent)
+
+
+def rule_matches(r1, r2, fields=('priority', 'iif', 'oif', 'dst', 'metric', 'proto'), verbose=False, indent=""):
+    return _matches(r1, r2, fields, verbose, indent)
+
+
+def _matches(r1, r2, fields, verbose, indent):
     for fld in fields:
         if verbose:
-            logger.debug("{}: {} - {}".format(fld, r1.get(fld), r2.get(fld)))
+            logger.debug("{}: {} - {}".format(fld, r1.get(fld), r2.get(fld)), extra={'iface': indent})
         if fld in r1 and fld in r2:
             if r1[fld] != r2[fld]:
                 return False
         elif fld in r1 or fld in r2:
-                return False
+            return False
     return True
 
+
 class RTLookup():
     def __init__(self, name):
         self.name = name
@@ -47,18 +59,20 @@ class RTLookup():
     def lookup_id(self, key):
         if type(key) == int or key.isdecimal():
             return int(key)
-        
+
         return self.str2id[key]
 
     def lookup_str(self, key):
         return self.id2str.get(key, key)
 
+
 class RTLookups():
     tables = RTLookup('rt_tables')
     realms = RTLookup('rt_realms')
     scopes = RTLookup('rt_scopes')
     protos = RTLookup('rt_protos')
 
+
 class Tables(collections.abc.Mapping):
     def __init__(self):
         self.tables = {
@@ -79,7 +93,7 @@ class Tables(collections.abc.Mapping):
 
     def add(self, route):
         rt = {
-            'table': RTLookups.tables.lookup_id( route.get('table', 254) ),
+            'table': RTLookups.tables.lookup_id(route.get('table', 254)),
             'dst': ip_network(route['to']).with_prefixlen,
             'scope': RTLookups.scopes.lookup_id(route.get('scope', 0)),
             'proto': RTLookups.protos.lookup_id(route.get('proto', 3)),
@@ -110,10 +124,11 @@ class Tables(collections.abc.Mapping):
                 continue
 
             if route['dst_len'] > 0:
-                dst = ip_network('{}/{}'.format(route.get_attr('RTA_DST'), route['dst_len'])).with_prefixlen
-            elif route['family']==AF_INET:
+                dst = ip_network(
+                    '{}/{}'.format(route.get_attr('RTA_DST'), route['dst_len'])).with_prefixlen
+            elif route['family'] == AF_INET:
                 dst = ip_network('0.0.0.0/0').with_prefixlen
-            elif route['family']==AF_INET6:
+            elif route['family'] == AF_INET6:
                 dst = ip_network('::/0').with_prefixlen
 
             rt = {
@@ -149,11 +164,16 @@ class Tables(collections.abc.Mapping):
     def kernel_routes(self, table):
         routes = []
         for route in ipr.get_routes(table=table, family=AF_INET) + ipr.get_routes(table=table, family=AF_INET6):
+            # ignore RTM_F_CLONED routes
+            if route['flags'] & 512:
+                continue
+
             if route['dst_len'] > 0:
-                dst = ip_network('{}/{}'.format(route.get_attr('RTA_DST'), route['dst_len'])).with_prefixlen
-            elif route['family']==AF_INET:
+                dst = ip_network(
+                    '{}/{}'.format(route.get_attr('RTA_DST'), route['dst_len'])).with_prefixlen
+            elif route['family'] == AF_INET:
                 dst = ip_network('0.0.0.0/0').with_prefixlen
-            elif route['family']==AF_INET6:
+            elif route['family'] == AF_INET6:
                 dst = ip_network('::/0').with_prefixlen
 
             rt = {
@@ -181,7 +201,7 @@ class Tables(collections.abc.Mapping):
             routes.append(rt)
         return routes
 
-    def apply(self, ignores):
+    def apply(self, ignores, do_apply):
         for table, croutes in self.tables.items():
             pfx = RTLookups.tables.lookup_str(table)
             logger.info('\nconfiguring routing table {}...'.format(pfx))
@@ -190,37 +210,50 @@ class Tables(collections.abc.Mapping):
 
             for route in croutes:
                 if 'oif' in route and type(route['oif']) == str:
-                    route['oif'] = next(iter(ipr.link_lookup(ifname=route['oif'])))
+                    route['oif'] = next(
+                        iter(ipr.link_lookup(ifname=route['oif'])))
                 found = False
                 identical = False
                 for i, kroute in enumerate(kroutes):
                     if route_matches(route, kroute):
                         del kroutes[i]
                         found = True
-                        if route_matches(route, kroute, route.keys(), True):
+                        if route_matches(route, kroute, route.keys(), True, route['dst']):
                             identical = True
                             break
 
                 if identical:
-                    logger.info('ok', extra={'iface': route['dst'], 'style': LogStyle.OK})
+                    logger.info(
+                        'ok', extra={'iface': route['dst'], 'style': LogStyle.OK})
                 else:
                     if found:
-                        logger.info('change', extra={'iface': route['dst'], 'style': LogStyle.CHG})
+                        logger.info('change', extra={
+                                    'iface': route['dst'], 'style': LogStyle.CHG})
                     else:
-                        logger.info('add', extra={'iface': route['dst'], 'style': LogStyle.CHG})
+                        logger.info(
+                            'add', extra={'iface': route['dst'], 'style': LogStyle.CHG})
 
-                    logger.debug("ip route replace: {}".format( " ".join("{}={}".format(k, v) for k,v in route.items()) ))
+                    logger.debug("ip route replace: {}".format(
+                        " ".join("{}={}".format(k, v) for k, v in route.items())))
                     try:
-                        ipr.route('replace', **route)
+                        if do_apply:
+                            ipr.route('replace', **route)
                     except NetlinkError as err:
-                        logger.warning('setup route {} failed: {}'.format(route['dst'], err.args[1]))
+                        logger.warning('route setup {} failed: {}'.format(
+                            route['dst'], err.args[1]))
 
             for route in kroutes:
                 if route['proto'] in ignores.get('protos', []):
                     continue
 
-                logger.info('del', extra={'iface': route['dst'], 'style': LogStyle.DEL})
-                ipr.route('del', **route)
+                logger.info(
+                    'del', extra={'iface': route['dst'], 'style': LogStyle.DEL})
+                try:
+                    if do_apply:
+                        ipr.route('del', **route)
+                except NetlinkError as err:
+                    logger.warning('removing route {} failed: {}'.format(
+                        route['dst'], err.args[1]))
 
 
 class Routes():
@@ -233,7 +266,8 @@ class Routes():
             raise RouteDupblicate()
 
         if 'table' in route:
-            route['table'] = RTLookups.tables.lookup_id(route.get_attr('RTA_TABLE', 'main'))
+            route['table'] = RTLookups.tables.lookup_id(
+                route.get_attr('RTA_TABLE', 'main'))
         # if 'dev' in route:
         #     route['dev'] = xxxx
         if 'scope' in route:
@@ -252,9 +286,118 @@ class Routes():
 
 
 class Rules():
+    def __init__(self):
+        self.rules = []
 
     def add(self, rule):
-        pass
+        ru = {
+            'table': RTLookups.tables.lookup_id(rule.get('table', 254)),
+            'protocol': RTLookups.protos.lookup_id(rule.get('proto', 0)),
+            'tos': rule.get('tos', 0),
+        }
 
-    def apply(self, ignores):
-        pass
+        if type(rule['action']) == str:
+            ru['action'] = {
+                "to_tbl": "FR_ACT_TO_TBL",
+                "unicast": "FR_ACT_UNICAST",
+                "blackhole": "FR_ACT_BLACKHOLE",
+                "unreachable": "FR_ACT_UNREACHABLE",
+                "prohibit": "FR_ACT_PROHIBIT",
+                "nat": "FR_ACT_NAT",
+            }.get(rule['action'])
+        else:
+            ru['action'] = rule.get('action'.lower(), "FR_ACTION_TO_TBL")
+
+        if 'fwmark' in rule:
+            ru['fwmark'] = rule['fwmark']
+
+        if 'ipproto' in rule:
+            if type(rule['ipproto']) == str:
+                ru['ip_proto'] = socket.getprotobyname(rule['ipproto'])
+            else:
+                ru['ip_proto'] = rule['ipproto']
+
+        if 'to' in rule:
+            ru['dst'] = str(ip_network(rule['to']).network_address)
+            ru['dst_len'] = ip_network(rule['to']).prefixlen
+
+        if 'from' in rule:
+            ru['src'] = str(ip_network(rule['from']).network_address)
+            ru['src_len'] = ip_network(rule['from']).prefixlen
+
+        if 'iif' in rule:
+            ru['iifname'] = rule['iif']
+
+        if 'oif' in rule:
+            ru['oifname'] = rule['oif']
+
+        if 'priority' in rule:
+            ru['priority'] = rule['priority']
+
+        self.rules.append(ru)
+
+    def kernel_rules(self):
+        rules = []
+        for rule in ipr.get_rules(family=AF_INET) + ipr.get_rules(family=AF_INET6):
+            ru = {
+                'action': FR_ACT_VALUES.get(rule['action']),
+                'table': rule.get_attr('FRA_TABLE'),
+                'protocol': rule.get_attr('FRA_PROTOCOL'),
+                'priority': rule.get_attr('FRA_PRIORITY', 0),
+                'family': rule['family'],
+                'tos': rule['tos'],
+            }
+
+            if rule['dst_len'] > 0:
+                ru['dst'] = rule.get_attr('FRA_DST')
+                ru['dst_len'] = rule['dst_len']
+
+            if rule['src_len'] > 0:
+                ru['src'] = rule.get_attr('FRA_SRC')
+                ru['src_len'] = rule['src_len']
+
+            for field in ['iifname', 'oifname', 'fwmark', 'ip_proto']:
+                value = rule.get_attr('FRA_{}'.format(field.upper()))
+                if not value is None:
+                    ru[field] = value
+
+            rules.append(ru)
+        return rules
+
+    def apply(self, ignores, do_apply):
+        logger.info('\nconfiguring routing rules...')
+        krules = self.kernel_rules()
+        for rule in self.rules:
+            found = False
+            for i, krule in enumerate(krules):
+                if rule_matches(rule, krule, rule.keys()):
+                    del krules[i]
+                    found = True
+                    break
+
+            if found:
+                logger.info(
+                    'ok', extra={'iface': '#{}'.format(rule['priority']), 'style': LogStyle.OK})
+            else:
+                logger.info(
+                    'add', extra={'iface': '#{}'.format(rule['priority']), 'style': LogStyle.CHG})
+
+                logger.debug("ip rule add: {}".format(
+                    " ".join("{}={}".format(k, v) for k, v in rule.items())))
+                try:
+                    if do_apply:
+                        ipr.rule('add', **rule)
+                except NetlinkError as err:
+                    logger.warning('rule setup failed: {}'.format(err.args[1]))
+
+        for rule in krules:
+            if rule['protocol'] in ignores.get('protos', []):
+                continue
+
+            logger.info(
+                'del', extra={'iface': '#{}'.format(rule['priority']), 'style': LogStyle.DEL})
+            try:
+                if do_apply:
+                    ipr.rule('del', **rule)
+            except NetlinkError as err:
+                logger.warning('removing rule failed: {}'.format(err.args[1]))
